
# q-expansion of newform 2352.2.a.y, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2352
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [4], [0], [4], [4], [0], [-4], [0], [2], [-8], [-6], [0], [-4], [8], [-10], [-4], [-4], [-4], [-8], [-16], [8], [12], [8], [8], [4], [8], [-4], [-14], [-14], [16], [12], [-10], [-12], [-10], [8], [4], [-12], [-8], [-4], [-12], [20], [0], [2], [6], [-8], [28], [16], [-20], [4], [-10], [24], [8], [20], [8], [-8], [28], [32], [22], [6], [28], [-12], [-20], [-32], [24], [30], [-20], [14], [-12], [-12], [-24], [-16], [16], [22], [-36], [-24], [-6], [4], [-18], [-24], [-28], [6], [-40], [-8], [0], [12], [-30], [-22], [12], [-8], [-20], [24], [8], [12], [12], [16], [-4], [32], [-4], [-2], [20], [-18], [4], [6], [-36], [-16], [-28], [-24], [-24], [-32], [0], [26], [22], [-20], [0], [-2], [36], [24], [-46], [-4], [28], [-34], [12], [44], [-20], [34], [-38], [24], [8], [-4], [20], [0], [0], [10], [16], [40], [36], [-20], [12], [42], [44], [-10], [-16], [-36], [-52], [-24], [52], [0], [36], [-48], [18], [-8], [-4], [-24], [20], [8], [48], [48], [0], [-12], [28], [-6], [8], [-4], [14], [8], [-8], [28], [-14], [12], [-36], [-12], [-24], [54], [-8], [8], [-44], [-6], [-16], [44], [0], [52], [-6], [-24], [40], [-52], [-34], [-36], [22], [40], [-32], [44], [-20], [-36], [68], [40], [46], [34], [40], [0], [14], [8], [28], [16], [-36], [-12], [40], [-36], [10], [68], [-18], [-12], [8], [-36], [-64], [40], [-40], [0], [24], [2], [58], [0], [-30], [-32], [-36], [-22], [-56], [8], [-48], [36], [18], [-44], [56], [-22], [12], [32], [56], [22], [-4], [-16], [44], [28], [-16], [14], [72], [24], [-32], [60], [-4], [32], [30], [40], [-72], [-16], [36], [-44], [-22], [4], [-60], [-8], [16], [44], [-20], [12], [48], [-28], [-18], [-72], [4], [-38], [-12], [20], [64], [32], [16], [-16], [52], [-2], [10], [-36], [16], [16], [40], [-20], [-20], [24], [-46], [54], [-56], [-72], [78], [-4], [6], [-4], [-18], [44], [-32], [28], [36], [52], [-40], [18], [56], [-84], [12], [-34], [52], [-68], [-48], [70], [48], [-54], [-66], [52], [8], [-80], [-28], [64], [80], [46], [20], [38], [-12], [-40], [-22], [72], [36], [-36], [24], [26], [-82], [66], [-24], [4], [52], [-68], [34], [-40], [64], [72], [86], [6], [-4], [0], [2], [12], [-60], [-52], [-48], [36], [-60], [74], [14], [8], [-10], [-56], [-72], [-52], [82], [64], [38], [-24], [72], [44], [28], [-42], [-84], [-56], [-70], [-12], [20], [-32], [86], [-32], [-2], [-4], [-40], [-32], [-80], [8], [-36], [-54], [-96], [98], [-76], [0], [24], [60], [4], [72], [-34], [20], [-68], [-44], [-64], [-26], [-72], [0], [4], [-54], [76], [34], [88], [-64], [-84], [8], [-50], [-78], [68], [-60], [64], [42], [-44], [44], [-42], [12], [0], [-56], [-16], [-48], [2], [92], [-48], [-12], [56], [20], [-36], [6], [52], [-40]]
